
;        globals
; Atari 2600 Equates

VSYNC   equ     $40             ;   0000 00x0       Vertical Sync Set-Clear
VBLANK  equ     $41             ;   xx00 00x0       Vertical Blank Set-Clear
WSYNC   equ     $42             ;   ---- ----       Wait for Horizontal Blank
RSYNC   equ     $43             ;   ---- ----       Reset Horizontal Sync Counter
NUSIZ0  equ     $44             ;   00xx 0xxx       Number-Size player/missle 0
NUSIZ1  equ     $45             ;   00xx 0xxx       Number-Size player/missle 1
COLUP0  equ     $46             ;   xxxx xxx0       Color-Luminance Player 0
COLUP1  equ     $47             ;   xxxx xxx0       Color-Luminance Player 1
COLUPF  equ     $48             ;   xxxx xxx0       Color-Luminance Playfield
COLUBK  equ     $49             ;   xxxx xxx0       Color-Luminance Background
CTRLPF  equ     $4A             ;   00xx 0xxx       Control Playfield, Ball, Collisions
REFP0   equ     $4B             ;   0000 x000       Reflection Player 0
REFP1   equ     $4C             ;   0000 x000       Reflection Player 1
PF0     equ     $4D             ;   xxxx 0000       Playfield Register Byte 0
PF1     equ     $4E             ;   xxxx xxxx       Playfield Register Byte 1
PF2     equ     $4F             ;   xxxx xxxx       Playfield Register Byte 2
RESP0   equ     $50             ;   ---- ----       Reset Player 0
RESP1   equ     $51             ;   ---- ----       Reset Player 1
RESM0   equ     $52             ;   ---- ----       Reset Missle 0
RESM1   equ     $53             ;   ---- ----       Reset Missle 1
RESBL   equ     $54             ;   ---- ----       Reset Ball
AUDC0   equ     $55             ;   0000 xxxx       Audio Control 0
AUDC1   equ     $56             ;   0000 xxxx       Audio Control 1
AUDF0   equ     $57             ;   000x xxxx       Audio Frequency 0
AUDF1   equ     $58             ;   000x xxxx       Audio Frequency 1
AUDV0   equ     $59             ;   0000 xxxx       Audio Volume 0
AUDV1   equ     $5A             ;   0000 xxxx       Audio Volume 1
GRP0    equ     $5B             ;   xxxx xxxx       Graphics Register Player 0
GRP1    equ     $5C             ;   xxxx xxxx       Graphics Register Player 1
ENAM0   equ     $5D             ;   0000 00x0       Graphics Enable Missle 0
ENAM1   equ     $5E             ;   0000 00x0       Graphics Enable Missle 1
ENABL   equ     $5F             ;   0000 00x0       Graphics Enable Ball
HMP0    equ     $60             ;   xxxx 0000       Horizontal Motion Player 0
HMP1    equ     $61             ;   xxxx 0000       Horizontal Motion Player 1
HMM0    equ     $62             ;   xxxx 0000       Horizontal Motion Missle 0
HMM1    equ     $63             ;   xxxx 0000       Horizontal Motion Missle 1
HMBL    equ     $64             ;   xxxx 0000       Horizontal Motion Ball
VDELP0  equ     $65             ;   0000 000x       Vertical Delay Player 0
VDELP1  equ     $66             ;   0000 000x       Vertical Delay Player 1
VDELBL  equ     $67             ;   0000 000x       Vertical Delay Ball
RESMP0  equ     $68             ;   0000 00x0       Reset Missle 0 to Player 0
RESMP1  equ     $69             ;   0000 00x0       Reset Missle 1 to Player 1
HMOVE   equ     $6A             ;   ---- ----       Apply Horizontal Motion
HMCLR   equ     $6B             ;   ---- ----       Clear Horizontal Move Registers
CXCLR   equ     $6C             ;   ---- ----       Clear Collision Latches
 
; Read Address Registers
                                                       ;		bit 7   bit 6
CXM0P   equ     $40             ;   xx00 0000       Read Collision  M0-P1   M0-P0
CXM1P   equ     $41             ;   xx00 0000                       M1-P0   M1-P1
CXP0FB  equ     $42             ;   xx00 0000                       P0-PF   P0-BL
CXP1FB  equ     $43             ;   xx00 0000                       P1-PF   P1-BL
CXM0FB  equ     $44             ;   xx00 0000                       M0-PF   M0-BL
CXM1FB  equ     $45             ;   xx00 0000                       M1-PF   M1-BL
CXBLPF  equ     $46             ;   x000 0000                       BL-PF   -----
CXPPMM  equ     $47             ;   xx00 0000                       P0-P1   M0-M1
INPT0   equ     $48             ;   x000 0000       Read Pot Port 0
INPT1   equ     $49             ;   x000 0000       Read Pot Port 1
INPT2   equ     $4A             ;   x000 0000       Read Pot Port 2
INPT3   equ     $4B             ;   x000 0000       Read Pot Port 3
INPT4   equ     $4C             ;   x000 0000       Read Input (Trigger) 0
INPT5   equ     $4D             ;   x000 0000       Read Input (Trigger) 1
 
; Atari 2600      RIOT Memory Map

; RAM equ $80                             ; 80-FF, also shadowed to $180-$1FF
 
SWCHA   equ     $280            ;                   Port A data register for joysticks:
							;					Bits 4-7 for player 1.  Bits 0-3 for player 2.
SWACNT  equ     $281            ;                   Port A data direction register (DDR)
SWCHB   equ     $282            ;                   Port B data (console switches)
SWBCNT  equ     $283            ;                   Port B DDR
INTIM   equ     $284            ;                   Timer output
TIM1T   equ     $294            ;                   set 1 clock interval
TIM8T   equ     $295            ;                   set 8 clock interval
TIM64T  equ     $296            ;                   set 64 clock interval
T1024T  equ     $297            ;                   set 1024 clock interval

